@@ -16,6 +16,9 @@
 #include <QPrintDialog>
 #include <thread>
 #include <QMessageBox>
+#ifdef Q_OS_WIN
+#include <QtPlatformHeaders\QWindowsWindowFunctions>
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // KiwixApp
@@ -81,18 +84,32 @@ void KiwixApp::init()
 
     createAction();
     mp_mainWindow = new MainWindow;
-    mp_tabWidget = mp_mainWindow->getTabBar();
-    mp_tabWidget->setContentManagerView(mp_manager->getView());
-    mp_tabWidget->setNewTabButton();
+    getTabWidget()->setContentManagerView(mp_manager->getView());
+    getTabWidget()->setNewTabButton();
     postInit();
     mp_errorDialog = new QErrorMessage(mp_mainWindow);
     setActivationWindow(mp_mainWindow);
     mp_mainWindow->show();
+#ifdef Q_OS_WIN
+    QWindow *window = mp_mainWindow->windowHandle();
+    if (window) {
+        QWindowsWindowFunctions::setHasBorderInFullScreen(window, true);
+    }
+#endif
     connect(this, &QtSingleApplication::messageReceived, this, [=](const QString &message) {
         if (!message.isEmpty()) {
             this->openZimFile(message);
         }
     });
+    connect(&m_watcher, &QFileSystemWatcher::directoryChanged, this, [=](QString monitorDir) {
+        m_library.asyncLoadMonitorDir(monitorDir);
+    });
+    QString monitorDir = m_settingsManager.getMonitorDir();
+    if (monitorDir != "") {
+        m_library.setMonitorDirZims(m_library.getLibraryZimsFromDir(monitorDir));
+        m_watcher.addPath(monitorDir);
+        m_library.asyncLoadMonitorDir(monitorDir);
+    }
 }
 
 KiwixApp::~KiwixApp()
@@ -186,14 +203,14 @@ void KiwixApp::openZimFile(const QString &zimfile)
 
 void KiwixApp::printPage()
 {
-    if(!mp_tabWidget->currentZimView())
+    if(!getTabWidget()->currentZimView())
         return;
     QPrinter* printer = new QPrinter();
     QPrintDialog printDialog(printer, mp_mainWindow);
     printDialog.setStyle(nullptr);
     printDialog.setStyleSheet("");
     if (printDialog.exec() == QDialog::Accepted) {
-        auto webview = mp_tabWidget->currentWebView();
+        auto webview = getTabWidget()->currentWebView();
         if(!webview)
             return;
         webview->page()->print(printer, [=](bool success) {
@@ -210,26 +227,29 @@ void KiwixApp::openUrl(const QString &url, bool newTab) {
 }
 
 void KiwixApp::openUrl(const QUrl &url, bool newTab) {
-    mp_tabWidget->openUrl(url, newTab);
+    getTabWidget()->openUrl(url, newTab);
 }
 
 void KiwixApp::openRandomUrl(bool newTab)
 {
-    auto zimId = mp_tabWidget->currentZimId();
+    auto zimId = getTabWidget()->currentZimId();
     if (zimId.isEmpty()) {
         return;
     }
-    auto reader = m_library.getReader(zimId);
+    
     try {
-        auto entry = reader->getRandomPage();
+        auto archive = m_library.getArchive(zimId);
+        auto entry = archive->getRandomEntry();
 
         QUrl url;
         url.setScheme("zim");
         url.setHost(zimId + ".zim");
         url.setPath("/" + QString::fromStdString(entry.getPath()));
         openUrl(url, newTab);
-    } catch ( const kiwix::NoEntry& ) {
+    } catch (const zim::EntryNotFound& e) {
         showMessage(gt("random-article-error"), gt("error-title"), QMessageBox::Information);
+    } catch (std::out_of_range& e) {
+        showMessage(gt("error-archive"), gt("error-title"), QMessageBox::Information);
     }
 }
 
@@ -265,6 +285,18 @@ bool KiwixApp::isCurrentArticleBookmarked()
     return false;
 }
 
+void KiwixApp::setMonitorDir(const QString &dir) {
+    m_settingsManager.setMonitorDir(dir);
+    m_library.setMonitorDirZims(QStringList());
+    for (auto path : m_watcher.directories()) {
+        m_watcher.removePath(path);
+    }
+    if (dir != "") {
+        m_watcher.addPath(dir);
+        m_library.asyncLoadMonitorDir(dir);
+    }
+}
+
 #define CREATE_ACTION(ID, TEXT) \
     mpa_actions[ID] = new QAction(TEXT)
 #define SET_SHORTCUT(ID, TEXT, SHORTCUT) \
@@ -354,7 +386,7 @@ void KiwixApp::createAction()
     CREATE_ACTION(FindInPageAction, gt("find-in-page"));
     mpa_actions[FindInPageAction]->setShortcuts({QKeySequence::Find, Qt::Key_F3});
     connect(mpa_actions[FindInPageAction], &QAction::triggered,
-            this, [=]() { mp_tabWidget->openFindInPageBar(); });
+            this, [=]() { getTabWidget()->openFindInPageBar(); });
 
     CREATE_ACTION_ICON_SHORTCUT(ToggleFullscreenAction, "full-screen-enter", gt("set-fullscreen"),  QKeySequence::FullScreen);
     connect(mpa_actions[ToggleFullscreenAction], &QAction::toggled,
@@ -399,11 +431,7 @@ void KiwixApp::createAction()
 }
 
 void KiwixApp::postInit() {
-    connect(mp_tabWidget, &TabBar::webActionEnabledChanged,
-            mp_mainWindow->getTopWidget(), &TopWidget::handleWebActionEnabledChanged);
-    connect(mp_tabWidget, &TabBar::currentTitleChanged, this,
-            [=](const QString& title) { emit currentTitleChanged(title); });
-    connect(mp_tabWidget, &TabBar::libraryPageDisplayed,
+    connect(getTabWidget(), &TabBar::libraryPageDisplayed,
             this, &KiwixApp::disableItemsOnLibraryPage);
     emit(m_library.booksChanged());
     connect(&m_library, &Library::booksChanged, this, &KiwixApp::updateNameMapper);
