@@ -9,7 +9,7 @@
 #include <QWebEngineSettings>
 #include <QWebEngineHistory>
 #include <QVBoxLayout>
-
+#include <zim/error.h>
 
 void WebViewBackMenu::showEvent(QShowEvent *)
 {
@@ -40,6 +40,29 @@ void WebViewForwardMenu::showEvent(QShowEvent *)
     setGeometry(geo);
 }
 
+QString getZimIdFromUrl(QUrl url)
+{
+    return url.host().split('.')[0];
+}
+
+QString getResultTypeFromUrl(QUrl url)
+{
+    return url.host().split('.')[1];
+}
+
+void WebView::applyCorrectZoomFactor() {
+    auto url = this->url();
+    auto settingsManager = KiwixApp::instance()->getSettingsManager();
+    qreal zoomFactor;
+    const bool isSearchResultsView = QUrlQuery(url).hasQueryItem("pattern") && (getResultTypeFromUrl(url) == "search");
+    if (isSearchResultsView) {
+        zoomFactor = settingsManager->getZoomFactor();
+    } else {
+        auto zimId = getZimIdFromUrl(url);
+        zoomFactor = settingsManager->getZoomFactorByZimId(zimId);
+    }
+    this->setZoomFactor(zoomFactor);
+}
 
 WebView::WebView(QWidget *parent)
     : QWebEngineView(parent)
@@ -49,6 +72,18 @@ WebView::WebView(QWidget *parent)
     connect(this->page(), &QWebEnginePage::linkHovered, this, [=] (const QString& url) {
         m_linkHovered = url;
     });
+
+    /* In Qt 5.12, the zoom factor is not correctly passed after a fulltext search
+     * Bug Report: https://bugreports.qt.io/browse/QTBUG-51851
+     * This rezooms the page to its correct zoom (default/by ZIM ID) after loading is finished.
+     * If the page is search results, we put the default zoom factor
+     * If in Qt 6.x, the bug is fixed this code can be removed.
+     */
+    connect(this, &QWebEngineView::loadFinished, this, [=] (bool ok) {
+        if (ok) {
+            applyCorrectZoomFactor();
+        }
+    });
 }
 
 WebView::~WebView()
@@ -125,25 +160,31 @@ QWebEngineView* WebView::createWindow(QWebEnginePage::WebWindowType type)
 }
 
 void WebView::onUrlChanged(const QUrl& url) {
-    auto zimId = url.host().split('.')[0];
+    auto zimId = getZimIdFromUrl(url);
     if (m_currentZimId == zimId ) {
         return;
     }
     m_currentZimId = zimId;
     emit zimIdChanged(m_currentZimId);
     auto app = KiwixApp::instance();
-    auto reader = app->getLibrary()->getReader(m_currentZimId);
-    if (!reader) {
+    std::shared_ptr<zim::Archive> archive;
+    try {
+        archive = app->getLibrary()->getArchive(m_currentZimId);
+    } catch (std::out_of_range& e) {
         return;
     }
-    std::string favicon, _mimetype;
-    reader->getFavicon(favicon, _mimetype);
-    QPixmap pixmap;
-    pixmap.loadFromData((const uchar*)favicon.data(), favicon.size());
-    m_icon = QIcon(pixmap);
-    emit iconChanged(m_icon);
     auto zoomFactor = app->getSettingsManager()->getZoomFactorByZimId(zimId);
     this->setZoomFactor(zoomFactor);
+    try {
+        std::string favicon, _mimetype;
+        auto item = archive->getIllustrationItem(48);
+        favicon = item.getData();
+        _mimetype = item.getMimetype();
+        QPixmap pixmap;
+        pixmap.loadFromData((const uchar*)favicon.data(), favicon.size());
+        m_icon = QIcon(pixmap);
+        emit iconChanged(m_icon);
+    } catch (zim::EntryNotFound& e) {}
 }
 
 void WebView::wheelEvent(QWheelEvent *event) {
